using CUDA
using .ExpressionProcessing
using .Interpreter

expressions = Vector{Expr}(undef, 2)
variables = Matrix{Float64}(undef, 2,2)
parameters = Vector{Vector{Float64}}(undef, 2)

# Resulting value should be 10 for the first expression
expressions[1] = :(x1 + 1 * x2 + p1)
expressions[2] = :(5 + x1 + 1 * x2 + p1 + p2)
variables[1,1] = 2.0
variables[2,1] = 3.0
variables[1,2] = 0.0
variables[2,2] = 5.0
parameters[1] = Vector{Float64}(undef, 1)
parameters[2] = Vector{Float64}(undef, 2)
parameters[1][1] = 5.0
parameters[2][1] = 5.0
parameters[2][2] = 0.0

@testset "Test interpretation" begin
	postfixExpr = expr_to_postfix(expressions[1])
	postfixExprs = Vector([postfixExpr])
	push!(postfixExprs, expr_to_postfix(expressions[2]))

	CUDA.@sync interpret(postfixExprs, variables, parameters)
end

@testset "Test conversion to matrix" begin
	reference = Matrix{Float64}(undef, 2, 2)
	reference[1,1] = 5.0
	reference[2,1] = NaN64
	reference[1,2] = 5.0
	reference[2,2] = 0.0
	# fill!(reference, [[], [5.0, 0.0]])
	# reference = Matrix([5.0, NaN],
	# 				   [5.0, 0.0])
	result = Interpreter.convert_to_matrix(parameters, NaN64)

	@test isequal(result, reference)
end

# TODO: Add several tests fo the mathematical expressions
#       One test for each operator. A second test if the operation order matters
#       And some more complicated expressions, with some only having variables, some only having parameters and some having both